@@ -52,12 +52,12 @@ under the same terms as Perl itself.
 use Exporter 'import';
 
 my @validators = qw(validate_path end_is_class b_is_subclass_of_a a_is_subclass_of_b root);
-our @EXPORT_OK = ( @validators, 'type_of', 'class_of', 'next_class');
+our @EXPORT_OK = ( @validators, 'type_of', 'class_of', 'next_class', 'resolve');
 our %EXPORT_TAGS = ( validate => \@validators );
 
 use strict;
 use InterMine::Model::Attribute;
-use Carp qw/confess/;
+use Carp qw/confess croak/;
 
 =head2 validate_path
 
@@ -100,11 +100,39 @@ sub validate_path {
 =cut
 
 sub last_bit {
-    my ( $model, $path_string ) = @_;
-    my @bits = _parse( $model, $path_string );
+    my ( $model, $path_string, $types) = @_;
+    my @bits = _parse( $model, $path_string, $types);
     return $bits[-1] || $bits[0];
 }
 
+sub last_bit_but_one {
+    my ( $model, $path_string, $types) = @_;
+    my @bits = _parse( $model, $path_string, $types);
+    return $bits[-2] || $bits[0];
+}
+
+sub last_class_type {
+    my ( $model, $path_string ) = @_;
+    my $end = last_bit_but_one( $model, $path_string );
+    if ( $end->isa('InterMine::Model::Reference') ) {
+        return $end->referenced_type_name;
+    } else {
+        return $end->name();    # because it's clearly a class
+    }
+}
+
+=head2 resolve
+
+Resolves a path to a class descriptor, or an attribute descriptor.
+
+=cut 
+
+sub resolve {
+    my ( $model, $string, $types) = @_;
+    my $bit = last_bit($model, $string, $types);
+    return class_of($bit) || $bit;
+}
+
 =head2 type_of
 
  Usage    : type_of($model, 'Department.name');
@@ -203,6 +231,8 @@ sub root {
 sub _parse {
     my ( $model, $path_string, $type_hashref ) = @_;
 
+    $type_hashref ||= {};
+
     # split Path.string into 'Path', 'string'
     my @bits = split /\./, $path_string;
     my @parts = ();    # <-- the classdescriptors will go here
@@ -239,7 +269,11 @@ sub _parse {
                         $bit,
                         $current_class->name(),
                     );
-                    confess $message;
+                    if ($ENV{DEBUG}) {
+                        confess $message;
+                    } else {
+                        croak $message;
+                    }
                 }
             }
             push @parts, $current_field;
