@@ -1,26 +1,25 @@
 package Webservice::InterMine::Query::Roles::Runnable;
 
-use Moose::Role;
-requires qw(view service url model);
+use MooseX::Role::WithOverloading;
+requires qw(view service model get_request_parameters resource_path
+            upload_path to_xml validate);
 
 use MooseX::Types::Moose qw(Str);
 use Perl6::Junction qw/any/;
-use JSON -support_by_pp, -no_export;
 use Webservice::InterMine::Service;
+use Webservice::InterMine::Types qw(ResultIterator);
 use Webservice::InterMine::ResultObject;
+use IO::Handle;
 
-has json_parser => (
-    is         => 'ro',
-    isa        => 'JSON',
-    lazy_build => 1,
+use constant {
+    DEFAULT_FORMAT => 'tab',
+};
+
+use overload (
+    '<>' => 'next_result',
+    fallback => 1,
 );
 
-sub _build_json_parser {
-    return JSON->new->allow_singlequote->allow_barekey;
-}
-
-use constant DEFAULT_FORMAT => 'tab';
-
 around BUILDARGS => sub {
     my $orig  = shift;
     my $class = shift;
@@ -37,6 +36,7 @@ around BUILDARGS => sub {
     }
 };
 
+
 =head1 NAME 
 
 Webservice::InterMine::Query::Roles::Runnable - Composable behaviour for runnable queries
@@ -51,35 +51,13 @@ This module provides composable behaviour for running a query against a webservi
 
 Returns a results iterator for use with a query.
 
-=cut
-
-sub results_iterator {
-    my $self  = shift;
-    my %args  = @_;
-    my $roles = delete $args{with};
-    return $self->service->get_results_iterator(
-        $self->url( %args ),
-        $self->view, $roles, 
-    );
-}
-
-my @json_format = (qw/jsonobjects jsonrows/);
-my @simple_formats = (qw/ string count /);
-my @valid_formats = (qw/ strings arrayref arrayrefs hashref hashrefs /, 
-    @json_format, @simple_formats);
-
-=head2 results( %options )
-
-returns the results from a query in the result format
-specified. 
-
 The following options are available:
 
 =over 4
 
 =item * as => $format
 
-Possible values: (string|strings|arrayrefs|hashrefs|jsonobjects|jsonrows|count)
+Possible values: (string|tsv|csv|arrayrefs|hashrefs|jsonobjects|jsonrows|count)
 
 The format to request results in. The default is C<arrayrefs>
 
@@ -91,6 +69,13 @@ The number of results to return. Leave undefined for "all" (default).
 
 The first result to return (starting at 0). The default is 0.
 
+=item * columnheaders => 0/1/friendly/path
+
+Whether to return the column headers at the top of TSV/CSV results. The default is
+false. There are two styles - friendly: "Gene > pathways > name" and 
+path: "Gene.pathways.name". The default style is friendly if a true value is entered and
+it is not "path".
+
 =item * json => $json_processor
 
 Possible values: (inflate|instantiate|raw|perl)
@@ -103,111 +88,140 @@ data structure. (default is C<perl>).
 
 =cut
 
-sub results {
-    my $self   = shift;
-    my %args   = @_;
-    my $wanted = $args{as} || 'arrayref';    # string and hashref are possible
-    confess "Invalid format selected" unless ($wanted eq any(@valid_formats));
+has _iterator => (
+    is => 'ro',
+    isa => ResultIterator,
+    lazy_build => 1,
+    builder => 'results_iterator',
+);
 
-    my $format = $self->DEFAULT_FORMAT;
-    if ( $wanted eq any(@json_format, 'count') ) {
-        $format = $wanted;
-        $wanted = 'string';
+sub next_result {
+    my $self = shift;
+    my $next = $self->_iterator->next;
+    $self->_clear_iterator unless (defined $next);
+    return $next;
+}
+
+sub results_iterator {
+    my $self  = shift;
+    my %args  = @_;
+
+    $self->validate;
+
+    my $row_format  = delete($args{as})   || "arrayrefs";
+    $row_format = 'tab' if ($row_format eq 'string' || $row_format eq 'tsv');
+    my $json_format = delete($args{json}) || "perl";
+    my $roles       = delete $args{with};
+
+    my %query_form = $self->get_request_parameters;
+    
+    # Set optional parameters
+    for my $opt (qw/start size columnheaders/) {
+        $query_form{$opt} = $args{$opt} if (defined $args{$opt});
     }
-        
-    $wanted =~ s/s$//;    # trim trailing 's' on arrayrefs/hashrefs
-    my $i     = $self->results_iterator(
-        format => $format,
-        start => $args{start},
-        size  => $args{size},
+    warn join(', ', map {"$_ => $query_form{$_}"} keys %query_form) if $ENV{DEBUG};
+    return $self->service->get_results_iterator(
+        $self->url,
+        \%query_form,
+        $self->view, 
+        $row_format, 
+        $json_format,
+        $roles, 
     );
-    my @lines = $i->all_lines($wanted);
-    if ( $wanted eq 'string' and $args{as} eq any(@simple_formats) ) {
-        return join( "\n", @lines );
-    }
-    elsif ( $wanted eq 'string' and $args{as} eq any(@json_format) ) {
-        $self->_handle_json_results( \@lines, lc( $args{json} ) );
-    }
-    else {
-        return \@lines;
-    }
 }
 
-##
-# _handle_json_results(\@lines, $json_format)
-#
-##
+=head2 results( %options )
 
-sub _handle_json_results {
-    my ( $self, $lines, $json ) = @_;
-    my $json_text = join( '', @$lines );
+returns the results from a query in the result format
+specified. 
 
-    return $json_text if ( $json eq 'raw' );
+The following options are available:
 
-    my $perl = $self->json_parser->decode($json_text)->{results};
+=over 4
 
-    if ( $json eq 'inflate' ) {
-        return inflate($perl);
-    }
-    elsif ( $json eq 'instantiate' ) {
-        return [ map { $self->model->make_new($_) } @$perl ];
-    }
-    else {
-        return $perl;
-    }
-}
+=item * as => $format
 
-=head1 FUNCTIONS
+Possible values: (tsv|csv|arrayrefs|hashrefs|jsonobjects|jsonrows|count)
 
-=head2 inflate( thing )
+The format to request results in. The default is C<arrayrefs>
 
-Inflates the thing passed in, blessing hashes into Webservice::InterMine::ResultObjects, 
-and recursing through their values, and iterating over their arrays.
+=item * size => $size
+
+The number of results to return. Leave undefined for "all" (default).
+
+=item * start => $start 
+
+The first result to return (starting at 0). The default is 0.
+
+=item * addheaders => 0/1/friendly/path
+
+Whether to return the column headers at the top of TSV/CSV results. The default is
+false. There are two styles - friendly: "Gene > pathways > name" and 
+path: "Gene.pathways.name". The default style is friendly if a true value is entered and
+it is not "path".
+
+=item * json => $json_processor
+
+Possible values: (inflate|instantiate|perl)
+
+What to do with JSON results. The results can be returned as inflated objects,
+full instantiated Moose objects, a raw json string, or as a perl
+data structure. (default is C<perl>).
+
+=back
 
 =cut
 
-sub inflate {
-    my $thing = shift;
-    my $type  = ref $thing;
-    if ( $type eq "HASH" ) {
-        bless $thing, "Webservice::InterMine::ResultObject";
-        for my $sub_thing ( values %$thing ) {
-            inflate($sub_thing);
-        }
-    }
-    elsif ( $type eq "ARRAY" ) {
-        for my $sub_thing (@$thing) {
-            inflate($sub_thing);
-        }
-    }
-    return $thing;
+sub results {
+    my $self = shift;
+    my $iter = $self->results_iterator(@_);
+    return $iter->get_all();
+}
+
+sub get_count {
+    my $self = shift;
+    my $iter = $self->results_iterator(as => "count");
+    return join('', $iter->get_all());
+}
+
+=head2 url 
+
+Get the url for a webservice resource.
+
+=cut
+
+sub url {
+    my $self = shift;
+    my $url = $self->service->root . $self->resource_path;
+    my $uri = URI->new($url);
+    return $uri;
+}
+
+sub get_upload_url {
+    my $self = shift;
+    my $url = return $self->service->root . $self->upload_path;
+    my $uri = URI->new($url);
+    return $uri;
 }
 
 sub save {
-
-    # saves queries as saved_queries, and templates as updated templates
-    confess "NOT IMPLEMENTED YET";
     my $self = shift;
     my %args = @_;
+    $self->validate;
     $self->name( $args{name} ) if ( exists $args{name} );
     my $xml  = $self->to_xml;
-    my $url  = $self->upload_url;
+    my $url  = $self->get_upload_url;
     my $resp = $self->service->send_off( $xml => $url );
-    confess "Failed to save data to webservice"
-      unless $resp->is_success;
+    my $name = $self->name;
+    my $root = $self->service->root;
+    unless ($resp =~ /$name\tsuccess/i) {
+        confess "Error saving $name to your account at $root:\n$resp";
+    }
     return;
 }
 
-sub save_as_template {    # saves queries as templates,
-                          # with all template constraint attributes
-                          # set to defaults values - can be used to save
-                          # a modified template under a new name
-    confess 'NOT IMPLEMENTED YET';    # TODO
-}
-
 1;
 
-
 =head1 SEE ALSO
 
 =over 4
